#!/usr/bin/env python3

import argparse
import re
import subprocess
import time

from urllib import request, parse

from selenium.webdriver import Firefox
from selenium.common.exceptions import NoSuchElementException

USER_AGENT = "Mozilla/5.0 (X11; Linux i686; rv:10.0) Gecko/20100101 Firefox/10.0"

# the versions that we consider useful to download, in order (better 1st)
VERSIONS = [
    re.compile(r"HDTV.*x264.*1080p"),
    re.compile(r".*x264.*1080p"),
    re.compile(r".*H.264.*1080p"),
    re.compile(r".*1080p"),
    re.compile(r"\sHDTV.*720p"),
    re.compile(r".*H264.*720p"),
    re.compile(r".*720p"),
    re.compile(r".*HDTV XviD"),
    re.compile(r".*BDRip x264  AAC"),
]


# set up selenium
browser = Firefox()


def sort_by_tag(info):
    """Get a release to download."""
    tag = info[0]
    for pos, regex in enumerate(VERSIONS):
        if regex.match(tag):
            break
    return pos


def get_data(url):
    """Get the movie/episode data."""
    print("Opening url")
    browser.get(url)

    # skip the notice text
    while True:
        print("Waiting for notice...")
        time.sleep(1)
        try:
            notice = browser.find_element_by_class_name("mode-notice")
        except NoSuchElementException:
            pass
        else:
            break
    print("Clicking in notice...")
    notice.find_element_by_class_name("link-button").click()

    # find how many episodes we have
    eplist = browser.find_element_by_class_name("episode-list")
    episodes_quantity = len(eplist.find_elements_by_tag_name("a"))

    all_episodes = []
    for episode_idx in range(episodes_quantity):
        # click on the given episode
        eplist = browser.find_element_by_class_name("episode-list")
        episode_items = eplist.find_elements_by_class_name("episode-item")
        episode_item = episode_items[episode_idx]
        episode_name = episode_item.text
        if "EXX" in episode_name:
            # generic all-season download
            continue
        episode_link = episode_item.find_element_by_tag_name("a")

        print("Clicking in episode", episode_idx + 1)
        episode_link.click()

        release_titles = browser.find_elements_by_class_name("release")
        release_infos = browser.find_elements_by_class_name("release-info")
        assert len(release_titles) == len(release_infos)

        data = []
        for r_title, r_info in zip(release_titles, release_infos):
            if not r_title.text.startswith("Grupo"):
                continue
            tag = r_title.find_element_by_class_name("release-tags").text

            magnet = subtitle = None
            for anchor in r_info.find_elements_by_tag_name("a"):
                link = anchor.get_attribute("href")
                if link.startswith("https://argenteam.net/subtitles/"):
                    subtitle = link
                elif link.startswith("magnet:"):
                    magnet = link
            data.append((tag, magnet, subtitle))

        all_episodes.append((episode_name, data))

    return all_episodes


def download_magnet(magnet_link):
    cmd = ['deluge-console', 'add', magnet_link]
    subprocess.run(cmd)


def download_subs(url):
    req = request.Request(url, headers={'User-Agent': USER_AGENT})
    resp = request.urlopen(req)
    filename = parse.urlsplit(resp.geturl()).path.lstrip('/')
    with open(filename, 'wb') as f:
        f.write(resp.read())


def download_episode(data, version, detached, only_subs, only_magnet):
    for tag, magnet_link, sub_link in data:
        if version is not None:
            tag_parts = set(tag.split())
            version_parts = set(version.split())
            if not version_parts.issubset(tag_parts):
                print("    Ignoring, bad version:", repr(tag))
                continue

        if magnet_link is None and not only_subs:
            print("    Ignoring, no magnet link:", repr(tag))
            continue

        if sub_link is None and not only_magnet:
            print("    Ignoring, no subs:", repr(tag))
            continue

        print("Got:", repr(tag))
        if not only_magnet:
            download_subs(sub_link)
        if not only_subs:
            download_magnet(magnet_link)
        return

    if not detached:
        print("Not found!!")
        return

    print("Attempting detached")
    got_subs = got_magnet = False
    for tag, magnet_link, sub_link in data:
        if not got_subs and sub_link is not None:
            print("    Subs from", repr(tag))
            download_subs(sub_link)
            got_subs = True
        if not got_magnet and magnet_link is not None:
            print("    Magnet from", repr(tag))
            download_magnet(magnet_link)
            got_magnet = True


def download_cross_versions(data, version):
    # get which is the first item with a subtitle
    all_with_subs = [x for x in data if x[2] is not None]
    if all_with_subs:
        top_with_sub = all_with_subs[0]
    else:
        print("Subtitle not found!!")
        top_with_sub = None

    for tag, magnet_link, sub_link in data:
        if version is not None and version not in tag:
            continue

        if magnet_link is not None:
            if sub_link is None:
                print("Found magnet from", repr(tag))
                if top_with_sub:
                    tag, _, sub_link = top_with_sub
                    print("Got sub from", repr(tag))
                    download_subs(sub_link)
            else:
                print("Found magnet and link from", repr(tag))
                download_subs(sub_link)
            return magnet_link

    print("Magnet not found!!")
    if top_with_sub:
        tag, _, sub_link = top_with_sub
        print("Got sub from", repr(tag))
        download_subs(sub_link)

    return None  # no magnet found


def main_classic(url, only_subs, version, detached, only_magnet):
    print("Getting metadata from Argenteam (only_subs={}, only_magnet={}, version={!r})".format(
        only_subs, only_magnet, version))
    all_episodes = get_data(url)

    for name, data in all_episodes:
        print("\nDownloading data for", repr(name))
        data.sort(key=sort_by_tag)
        download_episode(data, version, detached, only_subs, only_magnet)


def main_cross_versions(url, version):
    print("Getting metadata from Argenteam -- cross versions mode")
    all_episodes = get_data(url)
    magnet_fh = open("all_links.magnet", "wt")

    for name, data in all_episodes:
        print("\nDownloading data for", repr(name))
        data.sort(key=sort_by_tag)
        maybe_magnet = download_cross_versions(data, version)
        if maybe_magnet is not None:
            magnet_fh.write(f'deluge-console add "{maybe_magnet}"\n')

    magnet_fh.close()
    print("\nAll magnets links in all_links.magnet")


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("url")
    parser.add_argument("--only-subs", action='store_true')
    parser.add_argument("--only-magnet", action='store_true')
    parser.add_argument("--detached", action='store_true')
    parser.add_argument("--classic", action='store_true')
    parser.add_argument("--version")
    args = parser.parse_args()

    if args.classic:
        main_classic(args.url, args.only_subs, args.version, args.detached, args.only_magnet)
    else:
        main_cross_versions(args.url, args.version)
